// animation.h
// ===========
//
// Contains classes and methods for supporting sprite animation.
//
//----------------------------------------------------------------------------------------
//
// CDib class defines a Windows Device Independent Bitmap, or Dib Section.
//

#define ANIMATION

#define ERROR_COULD_NOT_FIND_RESOURCE        1001 
#define ERROR_COULD_NOT_LOAD_RESOURCE        1002 
#define ERROR_COULD_NOT_LOCK_RESOURCE        1003 

#define ERROR_COULD_NOT_ALLOCATE_MEMORY      1004 

#define ERROR_CANNOT_ALLOCATE_PALETTE        1100 
#define ERROR_CANNOT_CREATE_PALETTE          1101 
#define ERROR_HW_PALETTE_NOT_256_COLOURS     1102
#define ERROR_NOT_256_HW_PALETTE_ENTRIES     1103
#define ERROR_CANNOT_DELETE_PALETTE          1104

#define ERROR_PALETTIZATION_MISMATCH         1110
#define ERROR_CANNOT_REDUCE_TO_256_COLOURS   1111
#define ERROR_CANNOT_USE_PALETTIZED_DIB      1112
#define ERROR_PALETTE_IS_REQUIRED            1114

#define ERROR_NOT_256_COLOURS                1200 
#define ERROR_UNKNOWN_COLOUR_DEPTH           1201
#define ERROR_BAD_WIDTH_PARAM                1202
#define ERROR_BAD_HEIGHT_PARAM               1203

#define ERROR_BAD_TYPE                       1250

#define ERROR_PHASE_OUT_OF_RANGE             2000 

#define ERROR_RECTANGLE_OUT_OF_RANGE         2001

#define RGB_COLOUR           0
#define COLOUR_INDEX         1 
#define TOP_LEFT_PIXEL       2
#define TOP_RIGHT_PIXEL      3
#define BOTTOM_LEFT_PIXEL    4
#define BOTTOM_RIGHT_PIXEL   5

#define PALOP_NONE           0
#define PALOP_FOREGROUND     1
#define PALOP_BACKGROUND     2

#define LEFT_EDGE            1
#define TOP_EDGE             2
#define RIGHT_EDGE           3
#define BOTTOM_EDGE          4

class CDib
{
public :
	LPBITMAPINFOHEADER  pBmpInfoHdr ;

	BYTE *           pPixels  ;

	RGBQUAD *        pColourTable ;

	HBITMAP          hBmp ;     // only used for Dib Sections

	short            numOfColours ;

	short            width ;
	short            height ;
	short            bpp ;

	unsigned int     bgColour ;

	BYTE             bgPixel ;
	WORD             bgWord ;
	RGBTRIPLE        bgTriple ;

	short            errorCode ;

	bool             isDibSection ;

	bool             isPalettized ;

	bool             isResource ;

public :
	CDib() ;
	~CDib() ;

	void backcpy ( char * pDest , char * pSource , int num ) ;

	int getNumOfColours () ;
	int getBpp () ;
	BYTE * getPixelPtr () ;
	RGBQUAD * getColourTablePtr () ;

	int loadFromResource ( int resId , bool updateable = false ) ;

	void trimColours () ;

	int addColours ( CDib * pSourceDib ) ;

	int setBgColour ( unsigned int rgb , CPalette * pPal = NULL) ;

	int makePicture  (int width, int height , int * pColourTable = NULL ) ;

	int makePalette ( CPalette * pPal ) ;

	int palettize ( CPalette * pPal ) ;

	int makeDibSection ( int iWidth , int iHeight , int iBpp ,
		                 CDC * pDC ,
						 CPalette * pPal = NULL, bool palettizeIt = false ) ; 

	int bgFill ( CRect * pRect = NULL ) ;

	int solidMerge (   int  destXpos , int destYpos ,
		               LPBITMAPINFOHEADER psourceBmpInfoHdr ,
					   BYTE * psourceBmpPixels ,
					   int sourceXpos , int sourceYpos ,
					   int width ,
					   int height ,
					   bool reversed ) ;

	int transpMerge (  int  destXpos , int destYpos ,
		               LPBITMAPINFOHEADER psourceBmpInfoHdr ,
					   BYTE * psourceBmpPixels ,
					   int sourceXpos , int sourceYpos ,
					   int width ,
					   int height ,
					   bool reversed ,
					   unsigned int  transpPixel , bool inverted = false ) ;

	int maskMerge   ( int destXpos , int destYpos ,
		              LPBITMAPINFOHEADER psourceBmpInfoHdr ,
					  BYTE * psourceBmpPixels ,
					  int sourceXpos , int sourceYpos ,
					  int width ,
					  int height ,
					  char * mask ,
					  int maskWidth ,
					  int maskHeight ) ;

	int synchronize ( CDib * pSourceDib ) ;

	unsigned int getCornerPixel ( int location ) ;

	int draw    ( CRect &destRect , CRect &sourceRect , 
				  CDC * pDC , CDC * pMemDC ,
				  int palOp , CPalette * pPal ) ;

	int identityPalette ( CPalette * pPal , CDC * pDC ) ;

} ;
